#!/usr/bin/env node

//
// Meliora Testlab incoming/commit hook script
//
// Please refer to https://www.melioratestlab.com/mercurial-plugin on how to install this hook script.
//

// should be unique and somehow describe the repository you are pushing the information from
//
// note: If left blank, this gets set as basename of the hg root directory meaning, if you have
//  a repository in /some/path/myrepo, the identifier gets set as "myrepo". 
const REPOSITORY_ID = '';

////
//// For Meliora Testlab as a service:
////
const TESTLAB_COMPANY_ID = 'mycompany';
const TESTLAB_APIKEY = 'your testlab api key';
const TESTLAB_PROJECT = 'your testlab project prefix';
const TESTLAB_API_URL = 'https://' + TESTLAB_COMPANY_ID + '.melioratestlab.com/api/vcs/hg';

////
//// For Meliora Testlab as an on-premise install
////
//// Note: Please leave TESTLAB_COMPANY_ID as 'company', configure other values.
////
//const TESTLAB_COMPANY_ID = 'company';
//const TESTLAB_APIKEY = 'your testlab api key';
//const TESTLAB_PROJECT = your testlab project prefix';
//const TESTLAB_API_URL = 'http://yourtestlabserverhostandport/api/vcs/hg';

// uncomment __DEBUG_FILE to print out progress to specified log file
//const __DEBUG_FILE = '/tmp/testlab_hg_hook_debug.log';

//
// do not touch anything below this line
//

const HUMAN_READABLE_HG_LOG_STYLE = 'default';
const TEMPLATE_STYLES_FILE = '.hg/hg_testlab_hook.style';

const fs = require('fs');
const util = require('util');
const path = require('path');
const exec = require('child_process').exec;

var debugLogFile = null;
if(typeof __DEBUG_FILE !== 'undefined' && __DEBUG_FILE !== null) {
    debugLogFile = fs.createWriteStream(__DEBUG_FILE, { flags: 'a' });
}

var Client = require('node-rest-client').Client;
client = new Client({ user: TESTLAB_COMPANY_ID, password: TESTLAB_APIKEY });
client.registerMethod("put", TESTLAB_API_URL, "PUT");

function funcs() {
    this.exec = function(cmd, callback) {
        exec(cmd, (error, stdout, stderr) => {
            if(error) {
                var msg = 'exec failed: ' + error + '/' + stdout + '/' + stderr;
                this.debug(msg);
                console.error(msg);
                process.exit(1);
                return;
            }
            callback(stdout);
        });
    },
    this.debug = function() {
        if(debugLogFile != null) {
            debugLogFile.write(util.format.apply(null, arguments) + '\n');
        }
    }

}
const f = new funcs();

f.debug('== Running as ' + path.basename(__filename) + ' hook on', new Date());

const hgNode = process.env.HG_NODE;
if(hgNode) {
    createStylesIfMissing();
    if(!REPOSITORY_ID) {
        f.exec('hg root', function(hgRoot) {
            processHook(hgNode, path.basename(hgRoot).trim());
        });
    } else {
        processHook(hgNode, REPOSITORY_ID);
    }
}

function processHook(node, repositoryId) {
    f.debug('Processing for repository ' + repositoryId + ":", node);
    f.exec('hg log -r ' + hgNode  +' --style ' + HUMAN_READABLE_HG_LOG_STYLE, function(humanReadableLog) {
        f.exec('hg log -r ' + hgNode  +' --style .hg/hg_testlab_hook.style', function(log) {
            var data = {
                'project': TESTLAB_PROJECT,
                'repositoryId': repositoryId,
                'humanReadableLog': humanReadableLog,
                'log': log  
            };
            f.debug('Posting Changeset data to ' + TESTLAB_API_URL + ':', data);
            client.methods.put({
                headers: { "Content-Type": "application/json" },
                data: [ data ]
            }, function(data, response) {
                f.debug('Pushed 1 changeset to Testlab.\n');
            });
        });
    });
}

function createStylesIfMissing() {
    f.debug('Creating styles file to', TEMPLATE_STYLES_FILE);
    var s = fs.createWriteStream(TEMPLATE_STYLES_FILE);
    s.write('changeset = "node:{node}\\nrev:{rev}\\nparents:{parents}\\ntags:{tags}\\nauthor:{author}\\nbranches:{branches}\\ndate:{date}\\nfiles:{files}\\nfile_adds:{file_adds}\\nfile_dels:{file_dels}\\ndesc:\\n{desc}"\n');
    s.write('file = "{file},"\n');
    s.write('file_add = "{file_add},"\n');
    s.write('file_del = "{file_del},"\n');
    s.end();
}
